from __future__ import unicode_literals


from germanium.tools import assert_true, assert_equal
from germanium import config
from germanium.sugar import CSSMixin


class GermaniumAssertMixin(CSSMixin):

    def assert_present(self, selector, msg=None):
        if not msg:
            msg = 'Element %s is not present' % str(selector)
        return assert_true(self.wait_element_present(selector), msg)

    def assert_not_present(self, selector, msg=None):
        if not msg:
            msg = 'Element %s is present' % str(selector)
        return assert_true(self.wait_element_present(selector, False), msg)

    def assert_success(self, text):
        selector = ' '.join((config.FLASH, config.FLASH_TYPE['success'], config.FLASH_WRAPPER))
        self.wait_element_present(selector)
        self.assert_text(selector, text)

    def assert_visible_success(self):
        selector = ' '.join((config.FLASH, config.FLASH_TYPE['success'], config.FLASH_WRAPPER))
        self.wait_element_present(selector)
        self.assert_visible(selector)

    def assert_visible_loading(self):
        selector = ' '.join((config.FLASH, config.FLASH_TYPE['info'], config.FLASH_WRAPPER))
        self.wait_element_present(selector)
        self.assert_text(selector, 'Loading...')

    def assert_visible(self, selector):
        assert_true(self.wait_for_visible(selector), 'Element %s is not visible' % str(selector))

    def assert_not_visible(self, selector):
        assert_true(self.wait_for_visible(selector, False))

    def assert_text(self, selector, text):
        # Standard element.text has problem with html inside element
        html = self.css(selector).get_attribute("innerHTML")
        texts = selector, text, html
        assert_true(text in html, "Text does not match.\nSelector: %s\nExpected: %s\nActual: %s\n" % texts)

    def assert_dialog_present(self):
        self.assert_present(config.MODAL_DIALOG)

    def assert_current_path(self, path, msg=None):
        splitted_url = self.driver.current_url.split('/', 3)
        current_path = ''
        if len(splitted_url) == 4:
            current_path = '/' + splitted_url[3].split('?')[0].split('#')[0]

        assert_equal(path, current_path, msg)



from __future__ import unicode_literals

from django.utils.encoding import force_text

from six.moves.urllib.parse import urlencode

from .trivials import assert_equal, assert_in


def get_full_path(*paths):
    string_paths = (force_text(path) for path in paths)
    full_path = '/'.join((path[:-1] if path.endswith('/') else path for path in string_paths))
    return full_path if full_path.endswith('/') else full_path + '/'


def build_url(*paths, **querystring_dict):
    return (
        '{}?{}'.format(get_full_path(*paths), urlencode(querystring_dict))
        if querystring_dict else get_full_path(*paths)
    )


def assert_http_ok(resp, msg=None):
    """
    Ensures the response is returning a HTTP 200.
    """
    return assert_equal(resp.status_code, 200, resp.content if msg is None else msg)


def assert_http_created(resp, msg=None):
    """
    Ensures the response is returning a HTTP 201.
    """
    return assert_equal(resp.status_code, 201, resp.content if msg is None else msg)


def assert_http_accepted(resp, msg=None):
    """
    Ensures the response is returning a HTTP 202 or 204.
    """
    return assert_in(resp.status_code, [202, 204], resp.content if msg is None else msg)


def assert_http_multiple_choices(resp, msg=None):
    """
    Ensures the response is returning a HTTP 300.
    """
    return assert_equal(resp.status_code, 300, resp.content if msg is None else msg)


def assert_http_redirect(resp, msg=None):
    """
    Ensures the response is returning a HTTP 302.
    """
    return assert_equal(resp.status_code, 302, resp.content if msg is None else msg)


def assert_http_see_other(resp, msg=None):
    """
    Ensures the response is returning a HTTP 303.
    """
    return assert_equal(resp.status_code, 303, resp.content if msg is None else msg)


def assert_http_not_modified(resp, msg=None):
    """
    Ensures the response is returning a HTTP 304.
    """
    return assert_equal(resp.status_code, 304, resp.content if msg is None else msg)


def assert_http_bad_request(resp, msg=None):
    """
    Ensures the response is returning a HTTP 400.
    """
    return assert_equal(resp.status_code, 400, resp.content if msg is None else msg)


def assert_http_unauthorized(resp, msg=None):
    """
    Ensures the response is returning a HTTP 401.
    """
    return assert_equal(resp.status_code, 401, resp.content if msg is None else msg)


def assert_http_forbidden(resp, msg=None):
    """
    Ensures the response is returning a HTTP 403.
    """
    return assert_equal(resp.status_code, 403, resp.content if msg is None else msg)


def assert_http_not_found(resp, msg=None):
    """
    Ensures the response is returning a HTTP 404.
    """
    return assert_equal(resp.status_code, 404, resp.content if msg is None else msg)


def assert_http_method_not_allowed(resp, msg=None):
    """
    Ensures the response is returning a HTTP 405.
    """
    return assert_equal(resp.status_code, 405, resp.content if msg is None else msg)


def assert_http_conflict(resp, msg=None):
    """
    Ensures the response is returning a HTTP 409.
    """
    return assert_equal(resp.status_code, 409, resp.content if msg is None else msg)


def assert_http_gone(resp, msg=None):
    """
    Ensures the response is returning a HTTP 410.
    """
    return assert_equal(resp.status_code, 410, resp.content if msg is None else msg)


def assert_http_unprocessable_entity(resp, msg=None):
    """
    Ensures the response is returning a HTTP 422.
    """
    return assert_equal(resp.status_code, 422, resp.content if msg is None else msg)


def assert_http_too_many_requests(resp, msg=None):
    """
    Ensures the response is returning a HTTP 429.
    """
    return assert_equal(resp.status_code, 429, resp.content if msg is None else msg)


def assert_http_application_error(resp, msg=None):
    """
    Ensures the response is returning a HTTP 500.
    """
    return assert_equal(resp.status_code, 500, resp.content if msg is None else msg)


def assert_http_not_implemented(resp, msg=None):
    """
    Ensures the response is returning a HTTP 501.
    """
    return assert_equal(resp.status_code, 501, resp.content if msg is None else msg)


def assert_http_bad_gateway(resp, msg=None):
    """
    Ensures the response is returning a HTTP 502.
    """
    return assert_equal(resp.status_code, 502, resp.content if msg is None else msg)


def assert_http_service_unavailable(resp, msg=None):
    """
    Ensures the response is returning a HTTP 503.
    """

    return assert_equal(resp.status_code, 503, resp.content if msg is None else msg)


def assert_http_gateway_timeout(resp, msg=None):
    """
    Ensures the response is returning a HTTP 504.
    """
    return assert_equal(resp.status_code, 504, resp.content if msg is None else msg)



"""
Provides Nose and Django test case assert functions
"""

from __future__ import unicode_literals

from nose.tools import (assert_equal, assert_true, assert_false, assert_in, assert_not_in, assert_raises,
                        assert_not_equal, assert_is, assert_is_instance, assert_greater, assert_less,
                        assert_almost_equal, assert_not_almost_equal, assert_greater_equal, assert_less_equal,
                        assert_not_is_instance, assert_list_equal, assert_tuple_equal, assert_set_equal,
                        assert_dict_equal, assert_sequence_equal, assert_multi_line_equal, assert_is_none,
                        assert_is_not_none, assert_equals)


def fail(msg=None):
    raise AssertionError(msg)



from __future__ import unicode_literals

import collections
from functools import wraps

import six

import responses
from django.db import transaction
from django.db.models.fields import DateField, DateTimeField


def login(function=None, users_generator='get_user', **users_kwargs):
    """Login decorator usage: @login(user_data)"""

    def _login(function):
        def _decorator(self, *args, **kwargs):
            if isinstance(users_generator, six.string_types):
                users = getattr(self, users_generator)(**users_kwargs)
            else:
                users = users_generator(self, **users_kwargs)

            if not isinstance(users, (list, tuple)):
                users = (users,)

            for user in users:
                self.login(user)
                function(self, *args, **kwargs)
                self.logout()
        return wraps(function)(_decorator)

    if function:
        return _login(function)
    return _login


def login_all(cls=None, **user_kwargs):
    """Login decorator for all methods inside test usage: @login(user_data)"""

    def _login_all(cls):
        for attr, val in cls.__dict__.items():
            if callable(val) and attr.startswith("test_"):
                setattr(cls, attr, login(**user_kwargs)(val))
        return cls

    if cls:
        return _login_all(cls)
    return _login_all


def data_provider(fn_data_provider_or_str, *data_provider_args, **data_provider_kwargs):
    """Data provider decorator, allows another callable to provide the data for the test"""

    def test_decorator(fn):

        def get_data(self):
            if isinstance(fn_data_provider_or_str, six.string_types):
                provider_args = data_provider_args
                method_or_property = getattr(self, fn_data_provider_or_str)
            else:
                provider_args = (self,) + data_provider_args
                method_or_property = fn_data_provider_or_str

            return (
                method_or_property if not hasattr(method_or_property, '__call__')
                else method_or_property(*provider_args, **data_provider_kwargs)
            )

        def repl(self, *args):
            data = get_data(self)
            if not isinstance(data, collections .Iterable):
                data = (data,)
            for i in data:
                sid = transaction.savepoint()
                try:
                    if isinstance(i, collections.Iterable) and not isinstance(i, six.string_types):
                        fn(self, *i)
                    else:
                        fn(self, i)
                except AssertionError:
                    raise
                finally:
                    transaction.savepoint_rollback(sid)
                    responses.reset()
        return wraps(fn)(repl)
    return test_decorator


def turn_off_auto_now(model_class, field_name):

    def _turn_off_auto_now(function):
        def _decorator(self, *args, **kwargs):
                field = model_class._meta.get_field(field_name)
                if not isinstance(field, (DateField, DateTimeField)):
                    raise RuntimeError('Field %s must be DateField or DateTimeField type') % field_name
                if not field.auto_now:
                    raise RuntimeError('Field %s must have set auto_no to True') % field_name
                field.auto_now = False
                try:
                    function(self, *args, **kwargs)
                except AssertionError:
                    raise
                finally:
                    field.auto_now = True
        return wraps(function)(_decorator)

    return _turn_off_auto_now



from __future__ import unicode_literals

import types

from .default import GermaniumTestCase


def convert_to_test_obj(obj):
    obj.change_and_save = types.MethodType(change_and_save, obj)
    obj.reload = types.MethodType(reload, obj)
    return obj


def change_and_save(self, save_kwargs=None, **kwargs):
    save_kwargs = {} if save_kwargs is None else save_kwargs
    for attr, val in kwargs.items():
        setattr(self, attr, val)
    self.save(**save_kwargs)
    return self


def reload(self):
    if self.pk:
        self = self.__class__.objects.get(pk=self.pk)
    convert_to_test_obj(self)
    return self


class ModelTestCase(GermaniumTestCase):

    factory_class = None

    def inst_data_provider(self, **inst_kwargs):
        factory_class = inst_kwargs.pop('factory_class', self.factory_class)
        if 'pk' in inst_kwargs:
            inst = factory_class._get_model_class().objects.get(pk=inst_kwargs.get('pk'))
        else:
            inst = factory_class(**inst_kwargs)
        convert_to_test_obj(inst)
        return inst

    def insts_data_provider(self, count=10, **inst_kwargs):
        insts = []
        for _ in range(count):
            insts.append(self.inst_data_provider(**inst_kwargs))
        return insts



from __future__ import unicode_literals

import time

from selenium import webdriver
from selenium.common.exceptions import WebDriverException

from django.conf import settings
from django.contrib.staticfiles.testing import StaticLiveServerTestCase

from django_selenium import settings as selenium_settings
from django_selenium.testcases import MyDriver

from germanium import config
from germanium.patch import patch_broken_pipe_error

from .auth import AuthTestCaseMixin
from .default import GermaniumTestCaseMixin


SELENIUM_TESTS_WAIT = getattr(settings, 'SELENIUM_TESTS_WAIT', 0.1)
PHANTOM_JS_BIN = getattr(settings, 'PHANTOM_JS_BIN', None)


class ConfigurableWaitDriver(MyDriver):

    def __init__(self):
        driver = getattr(webdriver, selenium_settings.SELENIUM_DRIVER, None)
        assert driver, "settings.SELENIUM_DRIVER contains non-existing driver"
        if driver is webdriver.PhantomJS and PHANTOM_JS_BIN:
            self.driver = driver(PHANTOM_JS_BIN)
            self.live_server_url = 'http://%s:%s' % (selenium_settings.SELENIUM_TESTSERVER_HOST,
                                                     str(selenium_settings.SELENIUM_TESTSERVER_PORT))
            self.text = ''
        else:
            super(ConfigurableWaitDriver, self).__init__()

    def _wait_for_page_source(self):
        try:
            page_source = self.page_source
            time.sleep(SELENIUM_TESTS_WAIT)
            while page_source != self.page_source:
                page_source = self.page_source
                time.sleep(SELENIUM_TESTS_WAIT)
            self.update_text()
        except WebDriverException:
            pass


class GermaniumLiveServerTestCase(GermaniumTestCaseMixin, AuthTestCaseMixin, StaticLiveServerTestCase):

    is_logged = False

    @classmethod
    def set_up_class(cls):
        super(GermaniumLiveServerTestCase, cls).set_up_class()
        patch_broken_pipe_error()

    def tear_down(self):
        super(GermaniumLiveServerTestCase, self).tear_down()
        self.driver.quit()
        time.sleep(2)

    def set_up(self):
        super(GermaniumLiveServerTestCase, cls).set_up()
        self.driver = ConfigurableWaitDriver()
        self.driver.live_server_url = self.live_server_url
        self.driver.set_window_size(*config.WINDOW_SIZE)

    def open(self, url=''):
        self.driver.open_url(url)

    def logout(self):
        self.open(config.LOGOUT_URL)
        self.is_logged = False

    def authorize(self, username, password):
        self.open('/')
        self.type('input#id_' + config.USERNAME, username)
        self.type('input#id_' + config.PASSWORD, password)
        self.click(config.BTN_SUBMIT)
        time.sleep(1)



from __future__ import unicode_literals

from django.conf import settings


FLASH = getattr(settings, 'FLASH', '.flash')
FLASH_TYPE = getattr(settings, 'FLASH_TYPE', {
    'success': '.alert-success',
    'info': '.alert-info',
    'warning': '.alert-warn',
    'error': '.alert-danger'
})
FLASH_WRAPPER = getattr(settings, 'FLASH_WRAPPER', '.msg-text')
FLASH_ICON_CLOSE = getattr(settings, 'FLASH_ICON_CLOSE', '.goog-icon-remove')

USERNAME = getattr(settings, 'USERNAME', 'username')
PASSWORD = getattr(settings, 'PASSWORD', 'password')

BTN_SUBMIT = getattr(settings, 'BTN_SUBMIT', 'button[type="submit"]')
BTN_SAVE = getattr(settings, 'BTN_SAVE', '.btn-save')
BTN_SAVE_AND_CONTINUE = getattr(settings, 'BTN_SAVE_AND_CONTINUE', '.btn-save-and-continue')
BTN_CANCEL = getattr(settings, 'BTN_CANCEL', '.btn-cancel')

MODAL_DIALOG = getattr(settings, 'MODAL_DIALOG', '.modal-dialog')

SELENIUM_TESTS_WAIT = getattr(settings, 'SELENIUM_TESTS_WAIT', 1)
SELENIUM_DISPLAY_DIMENSION = getattr(settings, 'SELENIUM_DISPLAY_DIMENSION', (1440, 900))

SELENIUM_RUN_IN_BACKGROUND = getattr(settings, 'SELENIUM_RUN_IN_BACKGROUND', False)

LOGOUT_URL = getattr(settings, 'LOGOUT_URL', '/logout')
LOGIN_URL = getattr(settings, 'LOGIN_URL', '/login')

PHANTOM_JS_BIN = getattr(settings, 'PHANTOM_JS_BIN', 'phantomjs')
WINDOW_SIZE = (1124, 850)



from __future__ import unicode_literals


def patch_broken_pipe_error():
    """Monkey Patch BaseServer.handle_error to not write
    a stacktrace to stderr on broken pipe.
    http://stackoverflow.com/a/22618740/362702"""
    import sys
    from six.moves.socketserver import BaseServer
    from wsgiref import handlers

    handle_error = BaseServer.handle_error
    log_exception = handlers.BaseHandler.log_exception

    def is_broken_pipe_error():
        type, err, tb = sys.exc_info()
        return repr(err) == "error(32, 'Broken pipe')"

    def my_handle_error(self, request, client_address):
        if not is_broken_pipe_error():
            handle_error(self, request, client_address)

    def my_log_exception(self, exc_info):
        if not is_broken_pipe_error():
            log_exception(self, exc_info)

    BaseServer.handle_error = my_handle_error
    handlers.BaseHandler.log_exception = my_log_exception



from __future__ import unicode_literals

import json

from django.core.serializers.json import DjangoJSONEncoder
from django.test.client import MULTIPART_CONTENT

from germanium import config
from germanium.tools.http import assert_http_redirect

from .client import ClientTestCaseMixin
from .default import GermaniumTestCase, GermaniumSimpleTestCase


JSON_CONTENT_TYPE = 'application/json'


class RESTTestCaseMixin(ClientTestCaseMixin):

    SERIALIZERS = {
        JSON_CONTENT_TYPE: lambda data: json.dumps(data, cls=DjangoJSONEncoder),
        MULTIPART_CONTENT: lambda data: data,
    }

    DESERIALIZERS = {
        JSON_CONTENT_TYPE: lambda resp: json.loads(resp.content.decode('utf-8')),
        MULTIPART_CONTENT: lambda resp: resp,
    }

    def authorize(self, username, password):
        assert_http_redirect(self.post(config.LOGIN_URL, {config.USERNAME: username,
                                                          config.PASSWORD: password},
                                            content_type=MULTIPART_CONTENT))

    def get(self, url, content_type=None, headers=None):
        content_type = content_type or JSON_CONTENT_TYPE
        headers = headers or {}
        headers['Accept'] = headers.get('Accept', content_type)
        headers.update(self.default_headers)

        resp = self.c.get(url, content_type=content_type, **headers)
        return resp

    def put(self, url, data={}, content_type=None, headers=None):
        content_type = content_type or JSON_CONTENT_TYPE
        headers = headers or {}
        headers['Accept'] = headers.get('Accept', content_type)
        headers.update(self.default_headers)

        return self.c.put(url, data=self.serialize(data, content_type) if data is not None else data,
                          content_type=content_type, **headers)

    def post(self, url, data, content_type=None, headers=None):
        content_type = content_type or JSON_CONTENT_TYPE
        headers = headers or {}
        headers['Accept'] = headers.get('Accept', content_type)
        headers.update(self.default_headers)

        return self.c.post(url, data=self.serialize(data, content_type) if data is not None else data,
                           content_type=content_type, **headers)

    def patch(self, url, data, content_type=None, headers=None):
        content_type = content_type or JSON_CONTENT_TYPE
        headers = headers or {}
        headers['Accept'] = headers.get('Accept', content_type)
        headers.update(self.default_headers)

        return self.c.patch(url, data=self.serialize(data, content_type) if data is not None else data,
                           content_type=content_type, **headers)

    def delete(self, url, content_type=None, headers=None):
        content_type = content_type or JSON_CONTENT_TYPE
        headers = headers or {}
        headers['Accept'] = headers.get('Accept', content_type)
        headers.update(self.default_headers)

        resp = self.c.delete(url, **headers)
        return resp

    def deserialize(self, resp, content_type=None):
        """
        Given a ``HttpResponse`` coming back from using the ``client``, this method
        return dict of deserialized json string
        """
        content_type = content_type or JSON_CONTENT_TYPE
        deserializer = self.DESERIALIZERS.get(content_type)
        if deserializer is None:
            raise NotImplementedError('Missing DEserializer')
        else:
            return deserializer(resp)

    def serialize(self, data, content_type=None):
        """
        Given a Python datastructure (typically a ``dict``) & a desired  json,
        this method will return a serialized string of that data.
        """
        content_type = content_type or JSON_CONTENT_TYPE
        serializer = self.SERIALIZERS.get(content_type)
        if serializer is None:
            raise NotImplementedError('Missing serializer')
        else:
            return serializer(data)


class RESTTestCase(RESTTestCaseMixin, GermaniumTestCase):
    pass


class SimpleRESTTestCase(RESTTestCaseMixin, GermaniumSimpleTestCase):
    pass



from __future__ import unicode_literals

import warnings

warnings.warn('germanium.anotations module is deprecated, use annotations instead.', DeprecationWarning, stacklevel=2)

from germanium.annotations import *  # NOQA



from setuptools import setup, find_packages

setup(
    name='django-germanium',
    version='2.0.5',
    description='Helpful methods for Python Selenium and REST testing',
    author='Lukas Rychtecky, Lubos Matl',
    author_email='lukas.rychtecky@gmail.com, matllubos@gmail.com',
    packages=find_packages(),
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
    ],
    install_requires=[
        'django>=1.8',
        'django-selenium>=0.9.6',
        'PyVirtualDisplay>=0.1.2',
        'selenium>=2.37.2',
        'nose>=1.3.6',
        'responses>=0.5.1',
    ],
    include_package_data=True,
    zip_safe=False,
)
